﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Timers;
using System.Diagnostics;
using System.Reflection.Emit;
using System.Xml.Linq;
using TelemetryIO.Models;

namespace TelemetryIO
{
    public partial class MainForm : Form
    {
        private SerialHandler serial = new SerialHandler();
        private string[] speedItems = { "9600", "19200", "38400", "115200" };
        private System.Timers.Timer packageReceivingTimeout = new System.Timers.Timer(5);
        private System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();
        private System.Timers.Timer packageTransferTimer = new System.Timers.Timer(50);
        private List<byte> rx_buf = new List<byte>();
        private List<byte[]> send_queue = new List<byte[]>();
        private bool sendContinuousFlag = false;
        private float test_f = 0;

        public MainForm()
        {
            InitializeComponent();
            PortSelector.Items.AddRange(SerialPort.GetPortNames());
            PortSelector.SelectedIndex = 0;

            SpeedSelector.Items.AddRange(speedItems);
            SpeedSelector.SelectedIndex = 0;

            Send_btn.Enabled = false;

            packageTransferTimer.AutoReset = true;
            timer.Interval = 100;
            timer.Start();
            timer.Tick += Timer_Tick;

            ratePID_control.SubscribeOnEvent(serial);
            ratePID_control.bottom_label = InfoLabel;
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            if (name_label.InvokeRequired) {
                name_label.BeginInvoke(new Action(() => IMU_label.Text = Telemetry.Instance.name)); // Асинхронный вызов
            }
            else
            {
                name_label.Text = Telemetry.Instance.name;
            }
            UpdateLabel(serial.view_str);
        }

        private void serialSend()
        {

        }

        private void Send_btn_Click(object sender, EventArgs e)
        {
            serial.sendTest();
        }

        private void Connect_btn_Click(object sender, EventArgs e)
        {
            InfoLabel.Text = "";
            Button b = sender as Button;
            bool a = serial.isOpen;
            if (!serial.isOpen)
            {
                string port;
                string speed;
                try
                {
                    port = PortSelector.SelectedItem.ToString();
                    speed = SpeedSelector.SelectedItem.ToString();
                }
                catch (Exception ex)
                {
                    InfoLabel.Text = "Порт не существует.";
                    return;
                }

                if (port != null)
                {
                    try
                    {
                        serial.PortName = port;
                        serial.BaudRate = int.Parse(speed);
                        serial.Open();
                    }
                    catch (Exception ex)
                    {
                        InfoLabel.Text = ex.Message;
                        return;
                    }
                    b.Text = "Отключить";
                    Send_btn.Enabled = true;
                }
            }
            else
            {
                serial.Close();
                b.Text = "Подключить";
                Send_btn.Enabled = false;
            }

        }

        private void PortSelector_RefreshItems(object sender, MouseEventArgs e)
        {
            PortSelector.Items.Clear();
            PortSelector.Items.AddRange(SerialPort.GetPortNames());
            PortSelector.SelectedIndex = 0;
        }


        private void SendContinuous_btn_Click(object sender, EventArgs e)
        {
            serial.sendTestCont();
            /*Button b = sender as Button;
            if (sendContinuousFlag == true)
            {
                sendContinuousFlag = false;
                packageTransferTimer.Stop();

                b.Text = "Stop_Sending";
            }
            else
            {
                sendContinuousFlag = true;
                send_queue.Add(prepareTelegram());
                packageTransferTimer.Start();
                b.Text = "Send_Continuous";
            }*/
        }

        public void UpdateLabel(string newText)
        {

            if (IMU_label.InvokeRequired)
            {
                IMU_label.BeginInvoke(new Action(() => IMU_label.Text = newText)); // Асинхронный вызов
            }
            else
            {
                IMU_label.Text = newText;
            }
        }

        private void TestWrite_btn_Click(object sender, EventArgs e)
        {
            float[] f = {0, 1, 2, 3, 4, 5, 6, 7, 8};
            serial.sendTestWrite(f);
        }
    }
}
