﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace TelemetryIO.Models
{
    public sealed class Telemetry
    {
        private static volatile Telemetry instance;
        private static readonly object _lock = new object();

        public string name = "";
        public float[] raw_imu = new float[6];
        public float[] motor_act = new float[4];
        public float[] motor_sp = new float[4];
        public float[] pid0 = new float[9];
        public float[] pid1 = new float[9];
        public float[] pid2 = new float[9];
        public float[] pid3 = new float[9];

        private Telemetry() { }
        
        public static Telemetry Instance
        {
            get
            {
                if(instance == null)
                {
                    lock(_lock)
                    {
                        if (instance == null)
                        {
                            instance = new Telemetry();
                        }
                    }
                }
                return instance;
            }
        }

        public void setSlot(int slot, byte[] data, int offset, int len)
        {
            switch (slot)
            {
                case 0://RAW IMU
                    set_raw_imu(data, offset, len);
                    break;

                case 9999:
                    set_name(data);
                    break;
            }
        }
        private void set_raw_imu(byte[] data, int offset, int len)
        {
            for (int i = 0; i < (int)(len / 4); i++)
            {
                byte[] bs = new byte[4];
                Array.Copy(data, i * 4, bs, 0, 4);
                raw_imu[(int)(offset / 4)] = BitConverter.ToSingle(bs, 0);
            }
        }

        private void set_name(byte[] data)
        {
            name = Encoding.ASCII.GetString(data, 0, 16);
        }

    }
}
